require "schoolkeep"
require "faraday"

module Schoolkeep
  class Client
    attr_reader :api_key

    class InvalidTemplateName < StandardError; end

    def initialize(api_key)
      @api_key = api_key
    end

    def upload(name, path, engine)
      unless TEMPLATE_NAMES.include? name
        raise InvalidTemplateName.new("#{name} is an invalid template")
      end
      connection.post "/v1/custom_templates.json", {
        custom_template: {
          name: name,
          body: Faraday::UploadIO.new(path, 'text/plain; charset=utf-8'),
          engine: engine
        }
      }
    end

    def delete(name)
      unless TEMPLATE_NAMES.include? name
        raise InvalidTemplateName.new("#{name} is an invalid template")
      end
      connection.delete "/v1/custom_templates.json", {
        id: name
      }
    end

    def get_fixtures
      connection.get "/v1/fixtures.yaml"
    end

    private

    def connection
      @conenction ||= Faraday.new(
        url: Schoolkeep.api_url,
        ssl: { ca_file: File.join(GEM_ROOT, "config/ca-bundle.crt") }
      ) do |faraday|
        faraday.headers["X-Api-Key"] = api_key
        faraday.request :multipart
        faraday.request :url_encoded
        faraday.adapter Faraday.default_adapter
      end
    end
  end
end
