require "schoolkeep"
require "yaml"
require "schoolkeep/fixture/stubs"

module Schoolkeep
  class Fixture
    MAPPING = {
      activities: LearnerSyllabuses::Activity,
      apps: OpenStruct,
      color_palette: ColorScheme,
      commerce: BaseApp,
      course: SchoolWebsite::CourseDecorator,
      course_catalog: SchoolWebsite::CourseCatalogDecorator,
      courses: SchoolWebsite::CourseDecorator,
      current_learner: Learner,
      current_person: Person,
      current_school: School,
      current_student: Learner,
      custom_page: CustomPage,
      filterable_categories: CategoryDecorator,
      footer_navigations: WebsiteNavigation,
      header_navigations: WebsiteNavigation,
      homepage: Homepage,
      instructors: Partnerships::Educator,
      learner_syllabus: LearnerSyllabuses::Syllabus,
      multiple_languages: BaseApp,
      my_content: SchoolWebsite::MyContentDecorator,
      params: ActionController::Parameters,
      preview_banner: String,
      published_featured_courses: SchoolWebsite::CourseDecorator,
      published_sections: Section,
      routes: Schools::Routes,
      search_result: Search::CatalogResult,
      sections: LearnerSyllabuses::Section,
      social_media_links: SocialMediaLink,
      syllabus_activities: LearnerSyllabuses::Activity,
      website_footer: WebsiteFooter
    }

    KEY_MAPPING = {
      syllabus_activities: :activities
    }

    GEM_FIXTURE_PATH = File.join(GEM_ROOT, "config/fixtures.yml")

    attr_accessor :fixture_path

    def initialize(fixture_path)
      @fixture_path = fixture_path
    end

    def for(name, overrides: {})
      fixtures = uncasted_globals.merge(gem_fixtures["templates"][name] || {})

      if dev_fixtures["templates"]
        fixtures.merge!(dev_fixtures["templates"][name] || {})
      end

      fixtures.merge!(overrides)

      cast(fixtures).merge(template_name: File.basename(name, ".html.sktl"))
    end

    def globals
      cast(uncasted_globals)
    end

    private

    def uncasted_globals
      gem_fixtures["globals"].merge(dev_fixtures["globals"] || {})
    end

    def uncasted_models
      gem_fixtures["models"].merge(dev_fixtures["models"] || {})
    end

    def dev_fixtures
      @dev_fixtures ||= if File.exist?(fixture_path.to_s)
                          YAML.load_file(fixture_path.to_s)
                        else
                          {}
                        end
    end

    def gem_fixtures
      @gem_fixtures ||= YAML.load_file(GEM_FIXTURE_PATH)
    end

    def cast(values, key = nil)
      key &&= key.to_sym

      if values.is_a?(Symbol)
        values = uncasted_models[values.to_s]
      end

      if values.is_a? Array
        values = values.map do |value|
          cast(value, key)
        end
      else
        if values.is_a? Hash
          values = values.each_with_object({}) do |(child_key, value), hash|
            child_key &&= child_key.to_sym

            hash[KEY_MAPPING[child_key] || child_key] = cast(value, child_key)
          end
        end

        if MAPPING[key]
          values = MAPPING[key].new(values)
        end
      end

      values
    end

    class << self
      attr_accessor :default_path

      def for(name, overrides: {})
        new(default_path).for(name, overrides: overrides)
      end

      def globals
        new(default_path).globals
      end
    end
  end
end
