require "tk"
require "schoolkeep/server"

module Schoolkeep
  class GuiClient
    def initialize
      choose_templates_directory_button
      templates_directory_field
      server_toggle_button

      if not defined?(Ocra)
        Tk.mainloop
      end
    end

    def root
      @root ||= TkRoot.new do
        title "SchoolKeep Local Dev Server"
        minsize 400, 100
        iconphoto(
          *%w(144 72 57 16).map do |size|
            TkPhotoImage.new("file" => "media/sk-#{size}.gif")
          end
        )
      end
    end

    def server_toggle_button
      gui_client = self
      @server_toggle_button ||= TkButton.new(root) do
        text "Start Server"
        command do
          gui_client.toggle_server
        end
        pack
      end
    end

    def templates_directory_frame
      @templates_directory_frame ||= TkFrame.new(root) do
        pack "side" => "top"
      end
    end

    def templates_directory_field
      @templates_directory_field ||= TkEntry.new(templates_directory_frame) do
        pack "side" => "left"
      end
    end

    def choose_templates_directory_button
      gui_client = self
      @choose_templates_directory_button ||= TkButton.new(templates_directory_frame) do
        text "Choose Templates Directory"
        command do
          dir = gui_client.templates_directory_field.get
          if dir == ""
            dir = ENV["HOME"]
          end
          gui_client.templates_directory_field.set Tk::chooseDirectory("initialdir" => dir)
        end
        pack "side" => "right"
      end
    end

    def toggle_server
      if @server
        server_stop
      else
        server_start
      end
    end

    def server_start
      templates_directory_field.state "readonly"
      choose_templates_directory_button.state "disabled"
      @server = Thread.new {
        server = Server.new(
          dir: templates_directory_field.get,
          quiet: true
        )
        begin
            server.start
        ensure
            server.shutdown
        end
      }
      server_toggle_button.text "Stop Server"
    end

    def server_stop
      @server.kill
      @server = nil
      server_toggle_button.text "Start Server"
      templates_directory_field.state "normal"
      choose_templates_directory_button.state "normal"
    end
  end
end
