require "i18n"
require "scribble"
require "schoolkeep"
require "schoolkeep/fixture"

module Schoolkeep
  I18n.load_path << File.join(GEM_ROOT, "config/locale.yml")

  class Template
    CSS_MIME = "text/css"
    HTML_MIME = "text/html"

    class << self
      attr_accessor :path
    end

    attr_reader :name

    def initialize(name)
      @name = name
      @partial_loader = PartialLoader.new(self.class.path)
    end

    def source
      @source ||= File.read(
        File.join(self.class.path, name)
      )
    end

    def render(variables: {})
      template.render(
        variables: Fixture.for(name, overrides: variables)
      )
    end

    def type
      @type ||= case name
                when /\.css\.sktl$/
                  CSS_MIME
                when /\.html\.sktl$/
                  HTML_MIME
                end
    end

    private

    def template
      @template ||= ::Scribble::Template.new(
        source,
        loader: @partial_loader
      )
    end

    class PartialLoader
      attr_reader :path
      def initialize(path)
        @path = path
      end

      def load(name)
        name = "_#{name}.html.sktl"
        partial_path = File.join(path, name)
        ::Scribble::Partial.new File.read(partial_path)
      end
    end
  end
end
