require "erb"
require "schoolkeep"
require "schoolkeep/template"
require "schoolkeep/fixture"
require "schoolkeep/views/asset_cache"

module Schoolkeep
  module Views
    class Layout
      class << self
        attr_accessor :asset_host

        def asset_cache
          @asset_cache ||= AssetCache.new(asset_host)
        end
      end

      BODY_CLASSES = {
        "course_details.html.sktl" => "school_website-courses school_website-courses-show",
        "course_index.html.sktl" => "school_website-courses school_website-courses-index",
        "custom_page.html.sktl" => "school_website-custom_pages school_website-custom_pages-show",
        "homepage.html.sktl" => "school_website-homepages school_website-homepages-show",
        "my_content.html.sktl" => "school_website-my_content school_website-my_content-index"
      }

      LAYOUTS = Hash.new("school_website")
      LAYOUTS["course_cover.html.sktl"] = "learning"

      attr_reader :body
      def initialize(template)
        @template = template
        @school = Fixture.globals[:current_school]
        @preview_banner = Fixture.globals[:preview_banner]
      end

      def body_classes
        BODY_CLASSES[@template.name]
      end

      def asset_url(path)
        self.class.asset_cache[path]
      end

      def current_school
        @school
      end

      def preview_banner
        @preview_banner
      end

      def render_skt(partial_name, variables: {})
        name = "#{partial_name}.html.sktl"
        Template.new(name).render(variables: variables)
      end

      def render(variables: {})
        @body = @template.render(variables: variables)
        layout_path = File.join(GEM_ROOT, "config/#{LAYOUTS[@template.name]}.html.erb")
        layout= File.read(layout_path)
        set_context_variables_for_layout
        ERB.new(layout).result(binding)
      end

      private

      def set_context_variables_for_layout
        @display_search_form = @template.name == "my_content.html.sktl"
      end
    end
  end
end
